/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package ds3484test;

import java.net.*;
import java.io.*;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import javax.crypto.*;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;

public class DS3484Connection {

    private static final String KEY = "This MUST be 32 characters long.";

    private Socket ds_socket;
    private DataInputStream input;
    private DataOutputStream output;
    private NotificationInterface notify;
    private byte[] n_once = new byte[4];
    boolean connected = false;
    
    public DS3484Connection() {
        // Empty constructor
    }

    public DS3484Connection(String ip_address, int port) {
        connected = connect(ip_address, port);
    }
    
    public void registerForNotifications(NotificationInterface iface){
        notify = iface;
    }
    
    private boolean connect(String ip, int port) {
        // Try to connect to the module
        try {
            ds_socket = new Socket(ip, port);
            ds_socket.setSoTimeout(500);
        } catch (IOException e) {
            notify.networkError("Error connecting");
            return false;
        }
        // Get the tinput stream
        try {
            input = new DataInputStream(ds_socket.getInputStream());
        } catch (IOException e) {
            notify.networkError("Error connecting");
            return false;
        }
        // get the output stream
        try {
            output = new DataOutputStream(ds_socket.getOutputStream());
        } catch (IOException e) {
            notify.networkError("Error connecting");
            return false;
        }

        return true;        // No errors so return true
    }

    public void close() {
        connected = false;
        try {
            output.close();
            input.close();
            ds_socket.close();
        } catch (IOException e) {
            notify.networkError("Error closing connection");
        }
    }

    private void sendEncryptedMessage(byte[] command) {
        if (connected) {
            try {
                byte[] key_bytes = KEY.getBytes("UTF-8");                                   // Generate Key
                SecretKey key = new SecretKeySpec(key_bytes, "AES");

                byte[] iv = new byte[16];                                                   // Generate IV
                SecureRandom prng = new SecureRandom();
                prng.nextBytes(iv);

                Cipher aes = Cipher.getInstance("AES/CBC/PKCS5Padding");                    // Create cipher in CBC mode

                aes.init(Cipher.ENCRYPT_MODE, key, new IvParameterSpec(iv));

                byte[] encrypted = aes.doFinal(command);                                    // Encrypt the command

                byte[] packet = new byte[iv.length + encrypted.length];                     // This will be the the iv + encrypted message 
                System.arraycopy(iv, 0, packet, 0, iv.length);
                System.arraycopy(encrypted, 0, packet, iv.length, encrypted.length);

                output.write(packet, 0, packet.length);

            } catch (NoSuchAlgorithmException | NoSuchPaddingException | InvalidKeyException | IllegalBlockSizeException | BadPaddingException | InvalidAlgorithmParameterException | IOException ex) {
                notify.networkError("Encryption error check that AES key is correct");
            }
        }
    }

    private byte[] receiveAndDecryptMessage() {
        try {
            byte[] key_bytes = KEY.getBytes("UTF-8");                           // Generate Key
            SecretKey key = new SecretKeySpec(key_bytes, "AES");

            byte[] packet = receive();

            byte[] iv = new byte[16];                                           // Get the iv from the front of the packet 
            System.arraycopy(packet, 0, iv, 0, iv.length);

            byte[] data = new byte[16];
            System.arraycopy(packet, iv.length, data, 0, data.length);          // Get the packet from the packet after the iv

            Cipher aes = Cipher.getInstance("AES/CBC/NoPadding");               // Create cipher in CBC mode

            aes.init(Cipher.DECRYPT_MODE, key, new IvParameterSpec(iv));

            byte[] decrypted = aes.doFinal(data);                               // Decrypt the packet
            
            return decrypted;

        } catch (UnsupportedEncodingException | NoSuchAlgorithmException | NoSuchPaddingException | InvalidKeyException | InvalidAlgorithmParameterException | IllegalBlockSizeException | BadPaddingException ex) {
            notify.networkError("Encryption error check that AES key is correct");
        }

        return new byte[]{0};   // this will never be seen

    }

    private byte[] receive() {
        if (connected) {
           int bytes_read = 0;
            byte[] data = new byte[32];

            while (bytes_read < 32) // Read 32 bytes back form the module
            {
                try {
                    bytes_read += input.read(data, bytes_read, 32);
                } catch (IOException ex) {
                    notify.networkError("Read timeout, check d3484 is in AES Binary mode");
                    break;
                } catch (ArrayIndexOutOfBoundsException ex) {
                    notify.networkError("Read timeout, check d3484 is in AES Binary mode");
                    break;
                }
            }
            return data; 
        }
        return null;
    }
    
    public byte[] getStatus(){
        sendEncryptedMessage(new byte[]{0x30});             // Request board status
        byte[] packet = receiveAndDecryptMessage();
        System.arraycopy(packet, 12, n_once, 0, 4);         // Copy the random n_once number
        byte[] data = new byte[8];                      
        System.arraycopy(packet, 0, data, 0, 8);            // copy the data 
        return data;
    }
    
    public byte[] getRelayStates(byte relay){
        sendEncryptedMessage(new byte[]{0x33, relay});
        byte[] data = receiveAndDecryptMessage();
        byte[] relay_states = new byte[5];
        System.arraycopy(data, 0, relay_states, 0, 5);      // Copy the relay states to an array
        return relay_states;
    }
    
    public byte[] getIOs(byte io){
        sendEncryptedMessage(new byte[]{0x34, io});
        byte[] data = receiveAndDecryptMessage();
        byte[] input_states = new byte[2];
        System.arraycopy(data, 0, input_states, 0, 2);      // Copy the relay states to an array
        return input_states;
    }
    
    public void setRelay(byte relay_number, byte state){
        sendEncryptedMessage(new byte[] {0x31, relay_number, state, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, n_once[0], n_once[1], n_once[2], n_once[3]});
        byte[] data = receiveAndDecryptMessage();
        System.arraycopy(data, 12, n_once, 0, 4);            // Get the n_once random element
    }
    
    public void setOutput(byte output_number, byte state){
        sendEncryptedMessage(new byte[] {0x32, output_number, state, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, n_once[0], n_once[1], n_once[2], n_once[3]});
        byte[] data = receiveAndDecryptMessage();
        System.arraycopy(data, 12, n_once, 0, 4);            // Get the n_once random element
    }
    
    public int[] getAnalogueInputs(){
        
        sendEncryptedMessage(new byte[] {0x35} );
        byte[] data = receiveAndDecryptMessage();
        int[] values = new int[8];
        values[0] = ((data[0] & 0xff) << 8) | (data[1] & 0xff);     // Put the high and low bytes of the analogue values together
        values[1] = ((data[2] & 0xff) << 8) | (data[3] & 0xff);
        values[2] = ((data[4] & 0xff) << 8) | (data[5] & 0xff);
        values[3] = ((data[6] & 0xff) << 8) | (data[7] & 0xff);
        values[4] = ((data[8] & 0xff) << 8) | (data[9] & 0xff);
        values[5] = ((data[10] & 0xff) << 8) | (data[11] & 0xff);
        values[6] = ((data[12] & 0xff) << 8) | (data[12] & 0xff);
        values[7] = ((data[14] & 0xff) << 8) | (data[15] & 0xff);
        return values;
    }
    
    public int[] getCounterTimer(byte counter){
        int[] values = new int[2];
        sendEncryptedMessage(new byte[] {0x36, counter} );      // Command to get counter value
        byte data[] = receiveAndDecryptMessage();
        values[0] = ((data[0] & 0xff) << 24) | ((data[1] & 0xff) << 16) | ((data[2] & 0xff) << 8) | (data[3] & 0xff);
        values[1] = ((data[4] & 0xff) << 24) | ((data[5] & 0xff) << 16) | ((data[6] & 0xff) << 8) | (data[7] & 0xff);
        return values;
    }
}
