/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package dscriptmodbus;

import java.awt.Color;
import java.awt.event.WindowEvent;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JOptionPane;
import javax.swing.Timer;

/**
 * This is the UI for interacting with a dS2824
 * 
 * @author James Henderson
 */
public class DS2824UI extends ControlUI {

    private final ModbusTCPConnection connection;
    private final Timer timer;

    public DS2824UI(ModbusTCPConnection mtc) {
        initComponents();
        this.connection = mtc;
        
        // Set the connection that the ui controlls will use and the callback for errors
        ControlAction ca = new ControlAction();
        ca.setConnectionForCheckBoxes(mtc);
        ca.setErrorCallback((e) -> {
            this.alertError(e.getActionCommand());
        });

        // Set action for buttons
        this.relay1button.addActionListener(new RelayButtonAction((byte) 0));
        this.relay2button.addActionListener(new RelayButtonAction((byte) 1));
        this.relay3button.addActionListener(new RelayButtonAction((byte) 2));
        this.relay4button.addActionListener(new RelayButtonAction((byte) 3));
        this.relay5button.addActionListener(new RelayButtonAction((byte) 4));
        this.relay6button.addActionListener(new RelayButtonAction((byte) 5));
        this.relay7button.addActionListener(new RelayButtonAction((byte) 6));
        this.relay8button.addActionListener(new RelayButtonAction((byte) 7));
        this.relay9button.addActionListener(new RelayButtonAction((byte) 8));
        this.relay10button.addActionListener(new RelayButtonAction((byte) 9));
        this.relay11button.addActionListener(new RelayButtonAction((byte) 10));
        this.relay12button.addActionListener(new RelayButtonAction((byte) 11));
        this.relay13button.addActionListener(new RelayButtonAction((byte) 12));
        this.relay14button.addActionListener(new RelayButtonAction((byte) 13));
        this.relay15button.addActionListener(new RelayButtonAction((byte) 14));
        this.relay16button.addActionListener(new RelayButtonAction((byte) 15));
        this.relay17button.addActionListener(new RelayButtonAction((byte) 16));
        this.relay18button.addActionListener(new RelayButtonAction((byte) 17));
        this.relay19button.addActionListener(new RelayButtonAction((byte) 18));
        this.relay20button.addActionListener(new RelayButtonAction((byte) 19));
        this.relay21button.addActionListener(new RelayButtonAction((byte) 20));
        this.relay22button.addActionListener(new RelayButtonAction((byte) 21));
        this.relay23button.addActionListener(new RelayButtonAction((byte) 22));
        this.relay24button.addActionListener(new RelayButtonAction((byte) 23));
        this.relay25button.addActionListener(new RelayButtonAction((byte) 24));
        this.relay26button.addActionListener(new RelayButtonAction((byte) 25));
        this.relay27button.addActionListener(new RelayButtonAction((byte) 26));
        this.relay28button.addActionListener(new RelayButtonAction((byte) 27));
        this.relay29button.addActionListener(new RelayButtonAction((byte) 28));
        this.relay30button.addActionListener(new RelayButtonAction((byte) 29));
        this.relay31button.addActionListener(new RelayButtonAction((byte) 30));
        this.relay32button.addActionListener(new RelayButtonAction((byte) 31));

        // Set actiond for io check boxes
        this.ioCheckBox1.addActionListener(new IOCheckBoxAction((byte) 0));
        this.ioCheckBox2.addActionListener(new IOCheckBoxAction((byte) 1));
        this.ioCheckBox3.addActionListener(new IOCheckBoxAction((byte) 2));
        this.ioCheckBox4.addActionListener(new IOCheckBoxAction((byte) 3));
        this.ioCheckBox5.addActionListener(new IOCheckBoxAction((byte) 4));
        this.ioCheckBox6.addActionListener(new IOCheckBoxAction((byte) 5));
        this.ioCheckBox7.addActionListener(new IOCheckBoxAction((byte) 6));
        this.ioCheckBox8.addActionListener(new IOCheckBoxAction((byte) 7));

        // Set actions for the downstream relay buttons
        this.mbhButton1.addActionListener(new DownstreamRelayButtonAction((byte) 1));
        this.mbhButton2.addActionListener(new DownstreamRelayButtonAction((byte) 2));
        this.mbhButton3.addActionListener(new DownstreamRelayButtonAction((byte) 3));
        this.mbhButton4.addActionListener(new DownstreamRelayButtonAction((byte) 4));
        this.mbhButton5.addActionListener(new DownstreamRelayButtonAction((byte) 5));
        this.mbhButton6.addActionListener(new DownstreamRelayButtonAction((byte) 6));
        this.mbhButton7.addActionListener(new DownstreamRelayButtonAction((byte) 7));
        this.mbhButton8.addActionListener(new DownstreamRelayButtonAction((byte) 8));
        
        // Timer that will regularity grab the states of the relays and io
        timer = new Timer(100, (e) -> {
            setRelayControlStates();
        });

        timer.start();

    }

    /**
     * gets the relay and io states from the board and then sets the controls to
     * match those states
     */
    private void setRelayControlStates() {

        byte[] states;

        try {
            // Get the states of the relays and digital inputs back from the module
            // We request 48 points of information from the board witch will pack all of the states into it
            states = connection.getStates((byte) 48);
        } catch (IOException ex) {
            alertError("Failed to get the states of the relays ");
            return;
        }

        if ((states[9] & 0x01) == 0) {
            this.relay1button.setBackground(off);
        } else {
            this.relay1button.setBackground(on);
        }
        if ((states[9] & 0x02) == 0) {
            this.relay2button.setBackground(off);
        } else {
            this.relay2button.setBackground(on);
        }
        if ((states[9] & 0x04) == 0) {
            this.relay3button.setBackground(off);
        } else {
            this.relay3button.setBackground(on);
        }
        if ((states[9] & 0x08) == 0) {
            this.relay4button.setBackground(off);
        } else {
            this.relay4button.setBackground(on);
        }
        if ((states[9] & 0x10) == 0) {
            this.relay5button.setBackground(off);
        } else {
            this.relay5button.setBackground(on);
        }
        if ((states[9] & 0x20) == 0) {
            this.relay6button.setBackground(off);
        } else {
            this.relay6button.setBackground(on);
        }
        if ((states[9] & 0x40) == 0) {
            this.relay7button.setBackground(off);
        } else {
            this.relay7button.setBackground(on);
        }
        if ((states[9] & 0x80) == 0) {
            this.relay8button.setBackground(off);
        } else {
            this.relay8button.setBackground(on);
        }

        if ((states[10] & 0x01) == 0) {
            this.relay9button.setBackground(off);
        } else {
            this.relay9button.setBackground(on);
        }
        if ((states[10] & 0x02) == 0) {
            this.relay10button.setBackground(off);
        } else {
            this.relay10button.setBackground(on);
        }
        if ((states[10] & 0x04) == 0) {
            this.relay11button.setBackground(off);
        } else {
            this.relay11button.setBackground(on);
        }
        if ((states[10] & 0x08) == 0) {
            this.relay12button.setBackground(off);
        } else {
            this.relay12button.setBackground(on);
        }
        if ((states[10] & 0x10) == 0) {
            this.relay13button.setBackground(off);
        } else {
            this.relay13button.setBackground(on);
        }
        if ((states[10] & 0x20) == 0) {
            this.relay14button.setBackground(off);
        } else {
            this.relay14button.setBackground(on);
        }
        if ((states[10] & 0x40) == 0) {
            this.relay15button.setBackground(off);
        } else {
            this.relay15button.setBackground(on);
        }
        if ((states[10] & 0x80) == 0) {
            this.relay16button.setBackground(off);
        } else {
            this.relay16button.setBackground(on);
        }

        if ((states[11] & 0x01) == 0) {
            this.relay17button.setBackground(off);
        } else {
            this.relay17button.setBackground(on);
        }
        if ((states[11] & 0x02) == 0) {
            this.relay18button.setBackground(off);
        } else {
            this.relay18button.setBackground(on);
        }
        if ((states[11] & 0x04) == 0) {
            this.relay19button.setBackground(off);
        } else {
            this.relay19button.setBackground(on);
        }
        if ((states[11] & 0x08) == 0) {
            this.relay20button.setBackground(off);
        } else {
            this.relay20button.setBackground(on);
        }
        if ((states[11] & 0x10) == 0) {
            this.relay21button.setBackground(off);
        } else {
            this.relay21button.setBackground(on);
        }
        if ((states[11] & 0x20) == 0) {
            this.relay22button.setBackground(off);
        } else {
            this.relay22button.setBackground(on);
        }
        if ((states[11] & 0x40) == 0) {
            this.relay23button.setBackground(off);
        } else {
            this.relay23button.setBackground(on);
        }
        if ((states[11] & 0x80) == 0) {
            this.relay24button.setBackground(off);
        } else {
            this.relay24button.setBackground(on);
        }

        if ((states[12] & 0x01) == 0) {
            this.relay25button.setBackground(off);
        } else {
            this.relay25button.setBackground(on);
        }
        if ((states[12] & 0x02) == 0) {
            this.relay26button.setBackground(off);
        } else {
            this.relay26button.setBackground(on);
        }
        if ((states[12] & 0x04) == 0) {
            this.relay27button.setBackground(off);
        } else {
            this.relay27button.setBackground(on);
        }
        if ((states[12] & 0x08) == 0) {
            this.relay28button.setBackground(off);
        } else {
            this.relay28button.setBackground(on);
        }
        if ((states[12] & 0x10) == 0) {
            this.relay29button.setBackground(off);
        } else {
            this.relay29button.setBackground(on);
        }
        if ((states[12] & 0x20) == 0) {
            this.relay30button.setBackground(off);
        } else {
            this.relay30button.setBackground(on);
        }
        if ((states[12] & 0x40) == 0) {
            this.relay31button.setBackground(off);
        } else {
            this.relay31button.setBackground(on);
        }
        if ((states[12] & 0x80) == 0) {
            this.relay32button.setBackground(off);
        } else {
            this.relay32button.setBackground(on);
        }

        if ((states[14] & 0x01) == 0) {
            this.ioCheckBox1.setSelected(false);
        } else {
            this.ioCheckBox1.setSelected(true);
        }
        if ((states[14] & 0x02) == 0) {
            this.ioCheckBox2.setSelected(false);
        } else {
            this.ioCheckBox2.setSelected(true);
        }
        if ((states[14] & 0x04) == 0) {
            this.ioCheckBox3.setSelected(false);
        } else {
            this.ioCheckBox3.setSelected(true);
        }
        if ((states[14] & 0x08) == 0) {
            this.ioCheckBox4.setSelected(false);
        } else {
            this.ioCheckBox4.setSelected(true);
        }
        if ((states[14] & 0x10) == 0) {
            this.ioCheckBox5.setSelected(false);
        } else {
            this.ioCheckBox5.setSelected(true);
        }
        if ((states[14] & 0x20) == 0) {
            this.ioCheckBox6.setSelected(false);
        } else {
            this.ioCheckBox6.setSelected(true);
        }
        if ((states[14] & 0x40) == 0) {
            this.ioCheckBox7.setSelected(false);
        } else {
            this.ioCheckBox7.setSelected(true);
        }
        if ((states[14] & 0x80) == 0) {
            this.ioCheckBox8.setSelected(false);
        } else {
            this.ioCheckBox8.setSelected(true);
        }

        // Try and get the states of the relays from a downstream MBH88
        // If they cannot be fetched then disable the buttons and dont check again
        if (this.mbhButton1.isEnabled()) {
            
            try {

                byte[] stateOfDownstreamRelays = connection.getStateOfDownstreamRelays();

                if ((stateOfDownstreamRelays[9] & 0x01) == 0) {
                    this.mbhButton1.setBackground(off);
                } else {
                    this.mbhButton1.setBackground(on);
                }
                if ((stateOfDownstreamRelays[9] & 0x02) == 0) {
                    this.mbhButton2.setBackground(off);
                } else {
                    this.mbhButton2.setBackground(on);
                }
                if ((stateOfDownstreamRelays[9] & 0x04) == 0) {
                    this.mbhButton3.setBackground(off);
                } else {
                    this.mbhButton3.setBackground(on);
                }
                if ((stateOfDownstreamRelays[9] & 0x08) == 0) {
                    this.mbhButton4.setBackground(off);
                } else {
                    this.mbhButton4.setBackground(on);
                }
                if ((stateOfDownstreamRelays[9] & 0x10) == 0) {
                    this.mbhButton5.setBackground(off);
                } else {
                    this.mbhButton5.setBackground(on);
                }
                if ((stateOfDownstreamRelays[9] & 0x20) == 0) {
                    this.mbhButton6.setBackground(off);
                } else {
                    this.mbhButton6.setBackground(on);
                }
                if ((stateOfDownstreamRelays[9] & 0x40) == 0) {
                    this.mbhButton7.setBackground(off);
                } else {
                    this.mbhButton7.setBackground(on);
                }
                if ((stateOfDownstreamRelays[9] & 0x80) == 0) {
                    this.mbhButton8.setBackground(off);
                } else {
                    this.mbhButton8.setBackground(on);
                }

            } catch (IOException ex) {
                if (this.mbhButton1.isEnabled()) {
                    JOptionPane.showMessageDialog(this, "Could not get states of downstream relays, function will be disabled", "error", JOptionPane.WARNING_MESSAGE);
                    this.mbhButton1.setEnabled(false);
                    this.mbhButton2.setEnabled(false);
                    this.mbhButton3.setEnabled(false);
                    this.mbhButton4.setEnabled(false);
                    this.mbhButton5.setEnabled(false);
                    this.mbhButton6.setEnabled(false);
                    this.mbhButton7.setEnabled(false);
                    this.mbhButton8.setEnabled(false);
                }
            }
        }

    }


    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        relay25button = new javax.swing.JButton();
        relay26button = new javax.swing.JButton();
        relay27button = new javax.swing.JButton();
        relay28button = new javax.swing.JButton();
        relay29button = new javax.swing.JButton();
        relay30button = new javax.swing.JButton();
        relay31button = new javax.swing.JButton();
        relay17button = new javax.swing.JButton();
        relay18button = new javax.swing.JButton();
        relay1button = new javax.swing.JButton();
        relay19button = new javax.swing.JButton();
        relay2button = new javax.swing.JButton();
        relay3button = new javax.swing.JButton();
        relay4button = new javax.swing.JButton();
        relay5button = new javax.swing.JButton();
        relay6button = new javax.swing.JButton();
        relay7button = new javax.swing.JButton();
        relay8button = new javax.swing.JButton();
        relay20button = new javax.swing.JButton();
        relay21button = new javax.swing.JButton();
        relay16button = new javax.swing.JButton();
        relay22button = new javax.swing.JButton();
        relay9button = new javax.swing.JButton();
        relay23button = new javax.swing.JButton();
        relay10button = new javax.swing.JButton();
        relay11button = new javax.swing.JButton();
        relay12button = new javax.swing.JButton();
        relay13button = new javax.swing.JButton();
        relay14button = new javax.swing.JButton();
        relay15button = new javax.swing.JButton();
        relay24button = new javax.swing.JButton();
        relay32button = new javax.swing.JButton();
        ioCheckBox1 = new javax.swing.JCheckBox();
        ioCheckBox2 = new javax.swing.JCheckBox();
        ioCheckBox3 = new javax.swing.JCheckBox();
        ioCheckBox4 = new javax.swing.JCheckBox();
        ioCheckBox5 = new javax.swing.JCheckBox();
        ioCheckBox6 = new javax.swing.JCheckBox();
        ioCheckBox7 = new javax.swing.JCheckBox();
        ioCheckBox8 = new javax.swing.JCheckBox();
        jPanel1 = new javax.swing.JPanel();
        mbhButton1 = new javax.swing.JButton();
        mbhButton2 = new javax.swing.JButton();
        mbhButton3 = new javax.swing.JButton();
        mbhButton4 = new javax.swing.JButton();
        mbhButton5 = new javax.swing.JButton();
        mbhButton6 = new javax.swing.JButton();
        mbhButton7 = new javax.swing.JButton();
        mbhButton8 = new javax.swing.JButton();
        jLabel1 = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                formWindowClosing(evt);
            }
        });

        relay25button.setText("Relay 25");

        relay26button.setText("Relay 26");

        relay27button.setText("Relay 27");

        relay28button.setText("Relay 28");

        relay29button.setText("Relay 29");

        relay30button.setText("Relay 30");

        relay31button.setText("Relay 31");

        relay17button.setText("Relay 17");

        relay18button.setText("Relay 18");

        relay1button.setText("Relay 1");

        relay19button.setText("Relay 19");

        relay2button.setText("Relay 2");

        relay3button.setText("Relay 3");

        relay4button.setText("Relay 4");

        relay5button.setText("Relay 5");

        relay6button.setText("Relay 6");

        relay7button.setText("Relay 7");

        relay8button.setText("Relay 8");

        relay20button.setText("Relay 20");

        relay21button.setText("Relay 21");

        relay16button.setText("Relay 16");

        relay22button.setText("Relay 22");

        relay9button.setText("Relay 9");

        relay23button.setText("Relay 23");

        relay10button.setText("Relay 10");

        relay11button.setText("Relay 11");

        relay12button.setText("Relay 12");

        relay13button.setText("Relay 13");

        relay14button.setText("Relay 14");

        relay15button.setText("Relay 15");

        relay24button.setText("Relay 24");

        relay32button.setText("Relay 32");

        ioCheckBox1.setText("I/O 1");

        ioCheckBox2.setText("I/O 2");

        ioCheckBox3.setText("I/O 3");

        ioCheckBox4.setText("I/O 4");

        ioCheckBox5.setText("I/O 5");

        ioCheckBox6.setText("I/O 6");

        ioCheckBox7.setText("I/O 7");

        ioCheckBox8.setText("I/O 8");

        jPanel1.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));

        mbhButton1.setText("Relay 1");

        mbhButton2.setText("Relay 2");

        mbhButton3.setText("Relay 3");

        mbhButton4.setText("Relay 4");

        mbhButton5.setText("Relay 8");

        mbhButton6.setText("Relay 7");

        mbhButton7.setText("Relay 6");

        mbhButton8.setText("Relay 5");

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addGap(20, 20, 20)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(mbhButton8, javax.swing.GroupLayout.PREFERRED_SIZE, 104, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(mbhButton7, javax.swing.GroupLayout.PREFERRED_SIZE, 104, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(mbhButton6, javax.swing.GroupLayout.PREFERRED_SIZE, 104, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(mbhButton5, javax.swing.GroupLayout.PREFERRED_SIZE, 104, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(mbhButton1, javax.swing.GroupLayout.PREFERRED_SIZE, 104, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(mbhButton2, javax.swing.GroupLayout.PREFERRED_SIZE, 104, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(mbhButton3, javax.swing.GroupLayout.PREFERRED_SIZE, 104, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(mbhButton4, javax.swing.GroupLayout.PREFERRED_SIZE, 104, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(mbhButton1)
                    .addComponent(mbhButton2)
                    .addComponent(mbhButton3)
                    .addComponent(mbhButton4))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(mbhButton8)
                    .addComponent(mbhButton7)
                    .addComponent(mbhButton6)
                    .addComponent(mbhButton5))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        jLabel1.setText("Downstream MBH88 modbus module");

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addGroup(layout.createSequentialGroup()
                                        .addComponent(relay9button, javax.swing.GroupLayout.PREFERRED_SIZE, 106, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addGap(18, 18, 18)
                                        .addComponent(relay10button, javax.swing.GroupLayout.PREFERRED_SIZE, 106, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addGap(18, 18, 18)
                                        .addComponent(relay11button, javax.swing.GroupLayout.PREFERRED_SIZE, 106, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addGap(18, 18, 18)
                                        .addComponent(relay12button, javax.swing.GroupLayout.PREFERRED_SIZE, 106, javax.swing.GroupLayout.PREFERRED_SIZE))
                                    .addGroup(layout.createSequentialGroup()
                                        .addComponent(relay13button, javax.swing.GroupLayout.PREFERRED_SIZE, 106, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addGap(18, 18, 18)
                                        .addComponent(relay14button, javax.swing.GroupLayout.PREFERRED_SIZE, 106, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addGap(18, 18, 18)
                                        .addComponent(relay15button, javax.swing.GroupLayout.PREFERRED_SIZE, 106, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addGap(18, 18, 18)
                                        .addComponent(relay16button, javax.swing.GroupLayout.PREFERRED_SIZE, 106, javax.swing.GroupLayout.PREFERRED_SIZE)))
                                .addGroup(layout.createSequentialGroup()
                                    .addComponent(relay1button, javax.swing.GroupLayout.PREFERRED_SIZE, 106, javax.swing.GroupLayout.PREFERRED_SIZE)
                                    .addGap(18, 18, 18)
                                    .addComponent(relay2button, javax.swing.GroupLayout.PREFERRED_SIZE, 106, javax.swing.GroupLayout.PREFERRED_SIZE)
                                    .addGap(18, 18, 18)
                                    .addComponent(relay3button, javax.swing.GroupLayout.PREFERRED_SIZE, 106, javax.swing.GroupLayout.PREFERRED_SIZE)
                                    .addGap(18, 18, 18)
                                    .addComponent(relay4button, javax.swing.GroupLayout.PREFERRED_SIZE, 106, javax.swing.GroupLayout.PREFERRED_SIZE))
                                .addGroup(layout.createSequentialGroup()
                                    .addComponent(relay5button, javax.swing.GroupLayout.PREFERRED_SIZE, 106, javax.swing.GroupLayout.PREFERRED_SIZE)
                                    .addGap(18, 18, 18)
                                    .addComponent(relay6button, javax.swing.GroupLayout.PREFERRED_SIZE, 106, javax.swing.GroupLayout.PREFERRED_SIZE)
                                    .addGap(18, 18, 18)
                                    .addComponent(relay7button, javax.swing.GroupLayout.PREFERRED_SIZE, 106, javax.swing.GroupLayout.PREFERRED_SIZE)
                                    .addGap(18, 18, 18)
                                    .addComponent(relay8button, javax.swing.GroupLayout.PREFERRED_SIZE, 106, javax.swing.GroupLayout.PREFERRED_SIZE))
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addGroup(layout.createSequentialGroup()
                                        .addComponent(relay17button, javax.swing.GroupLayout.PREFERRED_SIZE, 106, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addGap(18, 18, 18)
                                        .addComponent(relay18button, javax.swing.GroupLayout.PREFERRED_SIZE, 106, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addGap(18, 18, 18)
                                        .addComponent(relay19button, javax.swing.GroupLayout.PREFERRED_SIZE, 106, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addGap(18, 18, 18)
                                        .addComponent(relay20button, javax.swing.GroupLayout.PREFERRED_SIZE, 106, javax.swing.GroupLayout.PREFERRED_SIZE))
                                    .addGroup(layout.createSequentialGroup()
                                        .addComponent(relay21button, javax.swing.GroupLayout.PREFERRED_SIZE, 106, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addGap(18, 18, 18)
                                        .addComponent(relay22button, javax.swing.GroupLayout.PREFERRED_SIZE, 106, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addGap(18, 18, 18)
                                        .addComponent(relay23button, javax.swing.GroupLayout.PREFERRED_SIZE, 106, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addGap(18, 18, 18)
                                        .addComponent(relay24button, javax.swing.GroupLayout.PREFERRED_SIZE, 106, javax.swing.GroupLayout.PREFERRED_SIZE))
                                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                        .addGroup(layout.createSequentialGroup()
                                            .addComponent(relay25button, javax.swing.GroupLayout.PREFERRED_SIZE, 106, javax.swing.GroupLayout.PREFERRED_SIZE)
                                            .addGap(18, 18, 18)
                                            .addComponent(relay26button, javax.swing.GroupLayout.PREFERRED_SIZE, 106, javax.swing.GroupLayout.PREFERRED_SIZE)
                                            .addGap(18, 18, 18)
                                            .addComponent(relay27button, javax.swing.GroupLayout.PREFERRED_SIZE, 106, javax.swing.GroupLayout.PREFERRED_SIZE)
                                            .addGap(18, 18, 18)
                                            .addComponent(relay28button, javax.swing.GroupLayout.PREFERRED_SIZE, 106, javax.swing.GroupLayout.PREFERRED_SIZE))
                                        .addGroup(layout.createSequentialGroup()
                                            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                                .addComponent(relay29button, javax.swing.GroupLayout.PREFERRED_SIZE, 106, javax.swing.GroupLayout.PREFERRED_SIZE)
                                                .addComponent(ioCheckBox1))
                                            .addGap(18, 18, 18)
                                            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                                .addComponent(relay30button, javax.swing.GroupLayout.PREFERRED_SIZE, 106, javax.swing.GroupLayout.PREFERRED_SIZE)
                                                .addComponent(ioCheckBox2)
                                                .addComponent(ioCheckBox6))
                                            .addGap(18, 18, 18)
                                            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                                .addComponent(relay31button, javax.swing.GroupLayout.PREFERRED_SIZE, 106, javax.swing.GroupLayout.PREFERRED_SIZE)
                                                .addComponent(ioCheckBox3)
                                                .addComponent(ioCheckBox7))
                                            .addGap(18, 18, 18)
                                            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                                .addComponent(ioCheckBox8)
                                                .addComponent(ioCheckBox4)
                                                .addComponent(relay32button, javax.swing.GroupLayout.PREFERRED_SIZE, 106, javax.swing.GroupLayout.PREFERRED_SIZE))))))
                            .addComponent(ioCheckBox5)
                            .addComponent(jLabel1))
                        .addGap(0, 0, Short.MAX_VALUE)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(relay1button)
                    .addComponent(relay2button)
                    .addComponent(relay3button)
                    .addComponent(relay4button))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(relay5button)
                    .addComponent(relay6button)
                    .addComponent(relay7button)
                    .addComponent(relay8button))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(relay9button)
                    .addComponent(relay10button)
                    .addComponent(relay11button)
                    .addComponent(relay12button))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(relay13button)
                    .addComponent(relay14button)
                    .addComponent(relay15button)
                    .addComponent(relay16button))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(relay17button)
                    .addComponent(relay18button)
                    .addComponent(relay19button)
                    .addComponent(relay20button))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(relay21button)
                    .addComponent(relay22button)
                    .addComponent(relay23button)
                    .addComponent(relay24button))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(relay25button)
                    .addComponent(relay26button)
                    .addComponent(relay27button)
                    .addComponent(relay28button))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(relay29button)
                    .addComponent(relay30button)
                    .addComponent(relay31button)
                    .addComponent(relay32button))
                .addGap(18, 18, 18)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(ioCheckBox1)
                    .addComponent(ioCheckBox2)
                    .addComponent(ioCheckBox3)
                    .addComponent(ioCheckBox4))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(ioCheckBox5)
                    .addComponent(ioCheckBox6)
                    .addComponent(ioCheckBox7)
                    .addComponent(ioCheckBox8))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(jLabel1)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    /**
     * Alert the user that an error has been hit and then close the application
     *
     * @param err
     */
    private void alertError(String err) {
        this.timer.stop();
        JOptionPane.showMessageDialog(this, err, "error", JOptionPane.WARNING_MESSAGE);
        dispatchEvent(new WindowEvent(this, WindowEvent.WINDOW_CLOSING));
    }

    /**
     * When the window closed stop timers and close the network connection
     *
     * @param evt
     */
    private void formWindowClosing(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosing
        this.timer.stop();
        try {
            this.connection.close();
        } catch (IOException ex) {
            Logger.getLogger(DS2824UI.class.getName()).log(Level.SEVERE, null, ex);
        }
    }//GEN-LAST:event_formWindowClosing


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JCheckBox ioCheckBox1;
    private javax.swing.JCheckBox ioCheckBox2;
    private javax.swing.JCheckBox ioCheckBox3;
    private javax.swing.JCheckBox ioCheckBox4;
    private javax.swing.JCheckBox ioCheckBox5;
    private javax.swing.JCheckBox ioCheckBox6;
    private javax.swing.JCheckBox ioCheckBox7;
    private javax.swing.JCheckBox ioCheckBox8;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JButton mbhButton1;
    private javax.swing.JButton mbhButton2;
    private javax.swing.JButton mbhButton3;
    private javax.swing.JButton mbhButton4;
    private javax.swing.JButton mbhButton5;
    private javax.swing.JButton mbhButton6;
    private javax.swing.JButton mbhButton7;
    private javax.swing.JButton mbhButton8;
    private javax.swing.JButton relay10button;
    private javax.swing.JButton relay11button;
    private javax.swing.JButton relay12button;
    private javax.swing.JButton relay13button;
    private javax.swing.JButton relay14button;
    private javax.swing.JButton relay15button;
    private javax.swing.JButton relay16button;
    private javax.swing.JButton relay17button;
    private javax.swing.JButton relay18button;
    private javax.swing.JButton relay19button;
    private javax.swing.JButton relay1button;
    private javax.swing.JButton relay20button;
    private javax.swing.JButton relay21button;
    private javax.swing.JButton relay22button;
    private javax.swing.JButton relay23button;
    private javax.swing.JButton relay24button;
    private javax.swing.JButton relay25button;
    private javax.swing.JButton relay26button;
    private javax.swing.JButton relay27button;
    private javax.swing.JButton relay28button;
    private javax.swing.JButton relay29button;
    private javax.swing.JButton relay2button;
    private javax.swing.JButton relay30button;
    private javax.swing.JButton relay31button;
    private javax.swing.JButton relay32button;
    private javax.swing.JButton relay3button;
    private javax.swing.JButton relay4button;
    private javax.swing.JButton relay5button;
    private javax.swing.JButton relay6button;
    private javax.swing.JButton relay7button;
    private javax.swing.JButton relay8button;
    private javax.swing.JButton relay9button;
    // End of variables declaration//GEN-END:variables
}
